from copy import deepcopy

from django.conf import settings
from django.db.models.fields import Field, CharField, TextField, FloatField
from django.utils import six
from django.utils.functional import lazy
from django.utils.translation import get_language

from . import CompositeField


LANGUAGES = map(lambda lang: lang[0], getattr(settings, 'LANGUAGES', ()))


class LocalizedField(CompositeField):

    def __init__(self, field_class, verbose_name=None, *args, **kwargs):
        self.languages = kwargs.pop('languages', LANGUAGES)
        if not self.languages:
            raise RuntimeError('Set LANGUAGES in your settings.py or pass a non empty "languages" argument before using LocalizedCharField')
        super(LocalizedField, self).__init__()
        self.verbose_name = verbose_name
        kwargs['verbose_name'] = verbose_name
        for language in self.languages:
            self[language] = field_class(*args, **kwargs)

    def contribute_to_class(self, cls, field_name):
        if self.verbose_name is None:
            self.verbose_name = field_name.replace('_', ' ')
        for language in self:
            # verbose_name must be lazy in order for the admin to show the
            # translated verbose_names of the fields
            self[language].verbose_name = lazy(lambda language: self.verbose_name + ' (' + language + ')', six.text_type)(language)
        super(LocalizedField, self).contribute_to_class(cls, field_name)

    def get_proxy(self, model):
        return LocalizedField.Proxy(self, model)

    def get_col(self, alias, field):
        current_field = self.current_field
        return current_field.get_col(alias, current_field)

    @property
    def current_field(self):
        language = get_language() or settings.LANGUAGE_CODE
        base_lang = language.split('-')[0]
        return self[base_lang]

    class Proxy(CompositeField.Proxy):

        def __bool__(self):
            return bool(six.text_type(self))

        def __unicode__(self):
            return unicode(self.current_with_fallback)

        def __setattr__(self, name, value):
            if name == 'current':
                language = get_language() or settings.LANGUAGE_CODE
                base_lang = language.split('-')[0]
                return setattr(self, base_lang, value)
            if name == 'all':
                for language in self._composite_field.languages:
                    setattr(self, language, value)
                return value
            else:
                return super(LocalizedField.Proxy, self).__setattr__(name, value)

        @property
        def current(self):
            language = get_language() or settings.LANGUAGE_CODE
            base_lang = language.split('-')[0]
            return getattr(self, base_lang)

        @property
        def current_with_fallback(self):
            language = get_language() or settings.LANGUAGE_CODE
            translation = None
            # 1. complete language code
            translation = getattr(self, language, None)
            if translation:
                return translation
            # 2. base of language code
            if '-' in language:
                base_lang = language.split('-')[0]
                translation = getattr(self, base_lang, None)
                if translation:
                    return translation
            # 3. first available translation
            for language in settings.LANGUAGES:
                base_lang = language[0].split('-')[0]
                translation = getattr(self, base_lang, None)
                if translation:
                    return translation
            return ''


class LocalizedCharField(LocalizedField):

    def __init__(self, *args, **kwargs):
        super(LocalizedCharField, self).__init__(CharField, *args, **kwargs)


class LocalizedTextField(LocalizedField):

    def __init__(self, *args, **kwargs):
        super(LocalizedTextField, self).__init__(TextField, *args, **kwargs)



from collections import OrderedDict
from copy import deepcopy

from django.db.models.fields import Field
from django.utils import six


class CompositeFieldBase(type):
    '''Metaclass for all composite fields.'''

    def __new__(cls, name, bases, attrs):
        super_new = super(CompositeFieldBase, cls).__new__
        # If this isn't a subclass of CompositeField, don't do anything special.
        if not any(isinstance(b, CompositeFieldBase) for b in bases):
            return super_new(cls, name, bases, attrs)

        # Prepare attributes.
        fields = []
        for field_name, field in list(attrs.items()):
            if hasattr(field, 'contribute_to_class'):
                fields.append((field_name, field))
                del attrs[field_name]
        fields.sort(key=lambda x: x[1].creation_counter)
        attrs['subfields'] = OrderedDict(fields)

        # Create the class.
        new_class = super_new(cls, name, bases, attrs)
        return new_class


@six.add_metaclass(CompositeFieldBase)
class CompositeField(object):
    is_relation = False
    concrete = False
    column = None
    rel = None  # Django<=1.9
    remote_field = None  # Django>=1.9
    auto_created = False
    editable = False
    serialize = False
    blank = True
    empty_values = []
    primary_key = False
    flatchoices = []

    def contribute_to_class(self, cls, name):
        self.name = name
        self.field_name = name
        self.attname = name
        # Only add the subfields for non-abstract models and use the model
        # attribute to detect non-abstract inheritance. Without this check
        # the subfields would be added multiple times.
        if not cls._meta.abstract and not self.model:
            self.model = cls
            if self.prefix is None:
                self.prefix = '%s_' % name
            for subfield_name, subfield in six.iteritems(self.subfields):
                subfield_name = self.prefix + subfield_name
                subfield.contribute_to_class(cls, subfield_name)
            setattr(cls, name, property(self.get, self.set))
        if hasattr(cls._meta, 'add_virtual_field'):
            # Django < 1.8
            cls._meta.add_virtual_field(self)
        else:
            cls._meta.add_field(self, virtual=True)

    def __init__(self, prefix=None):
        self.prefix = prefix
        self.model = None
        self.subfields = deepcopy(self.subfields)
        self.creation_counter = Field.creation_counter
        Field.creation_counter += 1
        for subfield in six.itervalues(self.subfields):
            subfield.creation_counter = Field.creation_counter
            Field.creation_counter += 1

    def __getitem__(self, name):
        return self.subfields[name]

    def __setitem__(self, name, subfield):
        self.subfields[name] = subfield

    def __contains__(self, name):
        return name in self.subfields

    def __iter__(self):
        return six.iterkeys(self.subfields)

    def __eq__(self, other):
        if isinstance(other, CompositeField):
            return self.creation_counter == other.creation_counter
        return NotImplemented

    def __lt__(self, other):
        if isinstance(other, CompositeField):
            return self.creation_counter < other.creation_counter
        return NotImplemented

    def __hash__(self):
        return hash(self.creation_counter)

    def get_proxy(self, model):
        return CompositeField.Proxy(self, model)

    def get(self, model):
        return self.get_proxy(model)

    def set(self, model, value):
        self.get_proxy(model)._set(value)

    def clean(self, value, model):
        return value

    def formfield(self, form):
        from django.forms import MultiValueField
        from django import forms
        return forms.CharField()

    class Proxy(object):

        def __init__(self, composite_field, model):
            object.__setattr__(self, '_composite_field', composite_field)
            object.__setattr__(self, '_model', model)

        def _subfield_name(self, name):
            if not name in self._composite_field:
                raise AttributeError('%r object has no attribute %r' % (
                        self._composite_field.__class__.__name__, name))
            return self._composite_field.prefix + name

        def _set(self, values):
            if isinstance(values, dict):
                for name in self._composite_field:
                    subfield_name = self._composite_field.prefix + name
                    setattr(self._model, subfield_name, values[name])
            else:
                for name in self._composite_field:
                    subfield_name = self._composite_field.prefix + name
                    setattr(self._model, subfield_name, getattr(values, name))

        def __setattr__(self, name, value):
            setattr(self._model, self._subfield_name(name), value)

        def __getattr__(self, name):
            return getattr(self._model, self._subfield_name(name))

        def __eq__(self, other):
            try:
                attrs_eq = [getattr(self, f) == getattr(other, f) for f in
                            self._composite_field]
            except AttributeError:
                return False
            return isinstance(other, self.__class__) and all(attrs_eq)

        def __ne__(self, other):
            return not self.__eq__(other)

        def __repr__(self):
            fields = ', '.join(
                '%s=%r' % (name, getattr(self, name))
                        for name in self._composite_field
            )
            return '%s(%s)' % (self._composite_field.__class__.__name__, fields)

        def to_dict(self):
            return {
                name: getattr(self, name)
                for name in self._composite_field
            }



#!/usr/bin/env python

import os
from setuptools import Command
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

# Prevent "TypeError: 'NoneType' object is not callable" when running tests.
# (http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

class DjangoTestCommand(Command):
    description = "run unit test using Django management command"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # Prepare DJANGO_SETTINGS_MODULE and PYTHONPATH
        import os, sys
        os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'
        sys.path[0:0] = [os.path.dirname(os.path.dirname(__file__))]
        # Execute management command 'test'
        from django.core.management import execute_from_command_line
        execute_from_command_line(['manage.py', 'test'])

setup(
    name='django-composite-field',
    version='0.7.4',
    description='CompositeField implementation for Django',
    long_description=read('README'),
    author='Michael P. Jung',
    author_email='michael.jung@terreon.de',
    license='BSD',
    keywords='django composite field',
    url='http://bitbucket.org/bikeshedder/django-composite-field',
    packages=['composite_field'],
    tests_require=['Django'],
    cmdclass={
        'test': DjangoTestCommand,
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
